package cn.iocoder.yudao.module.bpm.service.task;

import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.*;
import cn.iocoder.yudao.module.bpm.dal.dataobject.task.BpmTaskAssignRuleDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import cn.iocoder.yudao.module.bpm.dal.mysql.task.BpmTaskAssignRuleMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.*;

/**
 * Bpm 任务规则 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class BpmTaskAssignRuleServiceImpl implements BpmTaskAssignRuleService {

    @Resource
    private BpmTaskAssignRuleMapper taskAssignRuleMapper;

    @Override
    public Long createTaskAssignRule(BpmTaskAssignRuleSaveReqVO createReqVO) {
        // 插入
        BpmTaskAssignRuleDO taskAssignRule = BeanUtils.toBean(createReqVO, BpmTaskAssignRuleDO.class);
        taskAssignRuleMapper.insert(taskAssignRule);
        // 返回
        return taskAssignRule.getId();
    }

    @Override
    public void updateTaskAssignRule(BpmTaskAssignRuleSaveReqVO updateReqVO) {
        // 校验存在
        validateTaskAssignRuleExists(updateReqVO.getId());
        // 更新
        BpmTaskAssignRuleDO updateObj = BeanUtils.toBean(updateReqVO, BpmTaskAssignRuleDO.class);
        taskAssignRuleMapper.updateById(updateObj);
    }

    @Override
    public void deleteTaskAssignRule(Long id) {
        // 校验存在
        validateTaskAssignRuleExists(id);
        // 删除
        taskAssignRuleMapper.deleteById(id);
    }

    private void validateTaskAssignRuleExists(Long id) {
        if (taskAssignRuleMapper.selectById(id) == null) {
            throw exception(TASK_ASSIGN_RULE_NOT_EXISTS);
        }
    }

    @Override
    public BpmTaskAssignRuleDO getTaskAssignRule(Long id) {
        return taskAssignRuleMapper.selectById(id);
    }

    @Override
    public List<BpmTaskAssignRuleDO> getTaskAssignRuleModelId(String modelId) {
        return taskAssignRuleMapper.selectList(BpmTaskAssignRuleDO::getModelId, modelId);
    }

    @Override
    public PageResult<BpmTaskAssignRuleDO> getTaskAssignRulePage(BpmTaskAssignRulePageReqVO pageReqVO) {
        return taskAssignRuleMapper.selectPage(pageReqVO);
    }

}